% This code performs the calculations described in 
% the writeup, 'Stochastic Simulation of a Nonlinear, Dynamic Stochastic Model'
% by Chrisiano-Eichenbaum-Trabandt.

clear all;
close all;
dbstop if error

%&&& Parameters 
epsilon = 6; % elasticity of the final good producer's production function
beta    = .99; % discount factor

% Taylor
rhoR  = 0;% smoothing parameter on interest rate
phipi = 1.5; % response to inflation
phic  = 0; % respondse to output

% Calvo adjustment parameter
theta = .75;

% utility preference
sigma=1; % log utility function
psi = 1; % Frisch labor supply elasticity (= 1/psi).

% liquidity trap
rh = 1/beta-1;
rl = -.04/4; % negative natural rate
%uncomment the following line if you want no change in the discount rate
%rl = rh;
%this specifies how long the discount rate is perturbed (agents treat the
%discount rate as the realization of a two-state markov chain with the
%steady state as the absorbing state and the probability of rl remaining in
%place being q)
tp = 16; % tp is the last day when beta = beta_low;
q = .8; % Prob that the economy stays within ZLB
q = 0.8;

% gov.t. expenditure
g=.2;g=0;

if abs(g) < .1e-10
    kkap=(1-theta)*(1-beta*theta)/theta;
    den=((1-q)*(1-beta*q)-kkap*(psi+1)*q);
    piel=kkap*(psi+1)*beta*rl/den;
    yhatl=(beta*rl+q*piel)/(1-q);
end

%Agents believe that technology is a realization of the following
%stochastic process.
%to run the program you have to feed it a realization from this stochastic
%process. Agents see these realizations in real time, but they do not see
%them in advance. 
% Simulated path for the technology : a_path
% a(t+1) = (rho1 + rho2) * a(t) - rho1 * rho2 * a(t-1) + epsi.

rho1=0.95;
%rho1=.5;
rho2=0.2;
%rho2=0;

epsi = -.1;

%&&&end of parameters
a_path = nan(tp,1);
a_path(1) = epsi;
a_path(2) = (rho1 + rho2) * a_path(1);
for i = 3 : tp
    a_path(i) = (rho1 + rho2) * a_path(i-1) -rho1*rho2 * a_path(i-2);
end

%T is the date when the economy is presumed to return to steady state
T = tp + 100;
dt = 1 : T;
dt = dt';

% compute the continuation path of technology
for i = tp+1 : T
    a_path(i) = (rho1 + rho2) * a_path(i-1) -rho1*rho2 * a_path(i-2);
end
a_path_with_impulse=a_path;
% check that T is big enough that a_path has returned to zero
dist = norm(a_path(end));
if dist>6e-4
    error('Choose a bigger T');
end

% interest rate path (forecasted)
r_path = q.^dt * rl  + (1-q.^dt) *rh;


% Compute the steady state relation ship
[c_ss p_ss n_ss pi_ss K_ss F_ss R_ss w_ss] =ss_ZLB(beta,theta,g,psi);
Zss = [c_ss p_ss n_ss pi_ss K_ss F_ss R_ss];
Zss = Zss';
N   = length(Zss); % number of the endogenous varialbes


% Solve the model


% Set option for fzero
options=optimset('Display','iter','MaxFunEvals',1000000,'TolFun',1e-12,'MaxIter',1000);
Z_initial_guess = repmat(Zss,T,1); %#ok<NASGU>


% Prepare the box
Z_forecast = cell(T,1); % Agent's forecast.
Z_actual   = nan(N,T);  % The acual realization

% Simulation with impulse
tic;stochastic_simulation;toc
Z_impulse = Z_actual;
Z_forecast_impulse = Z_forecast;



% Simulation without impulse

% Reset
Z_forecast = cell(T,1); % Agent's forecast.
Z_actual   = nan(N,T);  % The acual realization
Z_initial_guess = repmat(Zss,T,1);
a_path = zeros(size(a_path));
r_path = q.^dt * rl  + (1-q.^dt) *rh;

% Do the stochastic simulation 
tic;stochastic_simulation;toc
Z_without_impulse = Z_actual;
Z_forecast_without_impulse = Z_forecast;

save results;


%% Plot the result with the impulse
close all;

ia=3;
ib=3;
Names = {'Consumption : C','Price dispersion : pistar','Labor : N','Inflation : \pi','K','F','Interest rate : R'};

   
II=[2 4 7];
figure('name','Stochastic Simulation With Impulse')
for ii = 1:7
    subplot(ia,ib,ii)
    if isempty(intersect(II,ii))
         plot(dt,Z_impulse(ii,:)/Zss(ii))
 %       plot(dt,Z_actual(ii,:))
    else
        plot(dt,Z_impulse(ii,:))
    end
    title(Names{ii})
    axis tight
    grid on
end

%% Plot the result without the impulse
%close all;
figure('name','Stochastic Simulation Without Impulse')


for ii = 1:7
    subplot(ia,ib,ii)
    if isempty(intersect(II,ii))
         plot(dt,Z_without_impulse(ii,:)/Zss(ii))
 %       plot(dt,Z_actual(ii,:))
    else
        plot(dt,Z_without_impulse(ii,:))
    end
    title(Names{ii} )
    axis tight
    grid on
end

%% Plot the results with and without tech shock
%close all
figure('name','With and without shock')
tx=1:40;
index = [1 4 7];
N1 = 2;
N2 = 2;
for ij = 1:length(index)
    ii=index(ij);
    subplot(N1,N2,ij)
    plot(dt(tx),Z_impulse(ii,tx),'*-',dt(tx),Z_without_impulse(ii,tx))
    if ij == 1
        legend('with technology','without technology','Location','SouthEast')
        %axis([tx(1) tx(end) .96 1.005])
    elseif ij == 2
        axis([tx(1) tx(end) .955 1.006])
    elseif ij == 3
        axis([tx(1) tx(end) .998 1.018])
    end
    title(Names{ii})
    grid on
end
subplot(N1,N2,length(index)+1)
plot(dt(tx),100*a_path_with_impulse(tx))
ylabel('percent')
title('technology shock')
axis tight
str='Figure: ZLB Episode With and Without Negative Technology Shock';
hout=suptitle(str);
orient landscape
print -dpdf 'with&without.pdf'

%% Plot the impulse response
%close all
figure('name','Impulse Response Function')
tx=1:40;
index = [1 4 7];
N1 = 2;
N2 = 2;
for ij = 1:length(index)
    ii=index(ij);
    subplot(N1,N2,ij)
    plot(dt(tx),100*log(Z_impulse(ii,tx)./Z_without_impulse(ii,tx)))
    ylabel('percent difference, pertubed versus non-perturbed')
    title(Names{ii})
    axis tight
    grid on
end
subplot(N1,N2,length(index)+1)
plot(dt(tx),100*a_path_with_impulse(tx))
ylabel('percent')
title('technology shock')
axis tight
grid on
orient landscape
print -dpdf 'impulse.pdf'


1+1;

